class SessionsController < ApplicationController
  before_action :set_session, except: [:start_demo]
  before_action :check_auth, only: [:setup_contracting, :check_existing_contract, :prepare_new_contract]

  def index
    render_bad_request unless @session
  end

  def start_demo
    begin
      @session = Session.start_for(params[:resource_id])
      redirect_to sessions_url(session_id: @session.id)
    rescue => e
      flash[:alert] = "Could not start session: #{e.message}"
      redirect_to '/'
    end
  end

  def own_creds_or_contract
    if params[:have_credentials] == 'true'
      @credential_fields = PlatformConnector.fields_for_credentials(@session.resource_id)
      render 'request_credentials'
    else
      if current_user
        @session.update(user: current_user)
        redirect_to setup_contracting_session_path(@session)
      else
        @user_session = UserSession.new
        @user = User.new
        @have_account = true
        render 'start_contracting'
      end
    end
  end

  def setup_contracting

  end

  def success
    render 'finish'
  end

  def check_existing_contract
    sleep 2
    first_contract = Contract.where(user_id: current_user.id, resource_id: @session.resource_id).first
    @contract_exists = first_contract.present?
    if @contract_exists
      @session.update(contract: first_contract)
      @session.make_ready!
    end
  end

  def prepare_new_contract
    Contract.prepare(@session)
    sleep 1
    @session.make_ready!
  end

  def use_own_creds
    @session.own_creds(params)

    render 'finish'
  end

  private

  def set_session
    @session = Session.find_by_id(params[:session_id]) || Session.find_by_id(params[:id])
  end

  def contract_params
    params[:credential_fields]
  end

  def invalid_credentials
    flash[:alert] = "Couldn't find a contract with given credentials"
    redirect_to credentials_url
  end
end