class Session < ActiveRecord::Base
  self.primary_key = :id

  belongs_to :contract
  belongs_to :user
  has_many :credential_fields

  validates :resource_id, presence: true

  include AASM

  aasm do
    state :initiating, initial: true
    state :ready

    event :make_ready do
      transitions from: :initiating, to: :ready
    end
  end

  def self.start_for(resource_id)
    Session.create!(id: SecureRandom.uuid, resource_id: resource_id, resource_name: PlatformConnector.resource_name_by_id(resource_id))
  end

  def own_creds(params)
    params[:credential_fields].each_pair do |cf_name, cf_value|
      CredentialField.write(self, cf_name, cf_value)
    end
    self.make_ready!
  end

  def cred_response
    if self.contract
      # User - ACSmI - CBP contract
      {session_id: self.id, contracting_type: 'platform',
       platform_credentials: {email: self.user.platform_email, password: self.user.platform_password}}
    else
      # User has provided own cloud credentials
      {session_id: self.id, contracting_type: 'own_credentials',
       resource_credentials: self.credential_fields.map { |cf| {name: cf.name, value: cf.value} }}
    end
  end

end