class PlatformConnector
  require 'rest_client'
  require 'rexml/document'

  def self.register_user(email, first_name, last_name, password)
    # use different route for ACSml user registration?
    response = create_request('user', {email: email, 'first_name' => first_name, 'last_name' => last_name,
                                       password: password, 'password_confirmation' => password, phone: '+48000000000',
                                       'user_role_name' => 'standard', limit: 5})
    response['user']['id']
  end

  def self.register_access(resource_id, user_id)
    create_request('access', { resource_id: resource_id, user_id: user_id, use_default_account: 'true', use_default_region: 'true' })
  end

  def self.fields_for_credentials(resource_id)
    response_to_hash(platform["/resources/#{resource_id}/fields_for_credentials"].get)['fields']
  end

  def self.resource_name_by_id(resource_id)
    begin
      response_to_hash(platform["/resources/#{resource_id}.xml"].get)['resource']['name']
    rescue
      raise "Resource with id #{resource_id} was not found"
    end
  end

  class << self

    private

    def platform
      RestClient::Resource.new(
          Setting.value('platform_url'),
          user: Setting.value('platform_email'),
          password: Setting.value('platform_password'),
          timeout: 60,
          open_timeout: 60,
          verify_ssl: true
      )
    end

    def create_request(model_class, attrs = {})
      begin
        response = platform["#{model_class.pluralize}/create.xml"].post(attrs.to_xml(root: model_class), content_type: :xml)
      rescue => e
        raise "Create #{model_class} failed: #{response} #{response ? response.body : 'no-response'}; exc: #{e.message}"
      end

      Hash.from_xml(response)
    end

    def response_to_hash(response)
      doc = REXML::Document.new(response.to_s)
      Hash.from_xml(doc.to_s)
    end
  end
end