/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.domain;

import java.io.Serializable;
import java.time.ZonedDateTime;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;

/**
 * A ServiceClass.
 */
@Entity
@Table(name = "service_class")
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class ServiceClass extends AbstractAuditingEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Size(max = 50)
    @Column(name = "serviceclassname", length = 50, nullable = false)
    private String serviceclassname;
    
    @NotNull
    @Column(name = "editable", nullable = false)
    private Boolean editable;

    @Column(name = "deleted_date")
    private ZonedDateTime deleteddate;
    
    @OneToMany(mappedBy = "serviceclass", fetch = FetchType.LAZY)
    @JsonIgnoreProperties({"serviceclass"})
    @Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
    private Set<Service> services = new HashSet<>();
    

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getServiceclassname() {
        return serviceclassname;
    }

    public ServiceClass serviceclassname(String serviceclassname) {
        this.serviceclassname = serviceclassname;
        return this;
    }

    public void setServiceclassname(String serviceclassname) {
        this.serviceclassname = serviceclassname;
    }
    
    public Boolean isEditable() {
        return editable;
    }
    
    public void setEditable(boolean editable) {
    	this.editable = editable;
    }

    public ZonedDateTime getDeleteddate() {
        return deleteddate;
    }

    public ServiceClass deleteddate(ZonedDateTime deleteddate) {
        this.deleteddate = deleteddate;
        return this;
    }

    public void setDeleteddate(ZonedDateTime deleteddate) {
        this.deleteddate = deleteddate;
    }
    
    public Set<Service> getServices() {
        return services;
    }

    public ServiceClass services(Set<Service> services) {
        this.services = services;
        return this;
    }

    public ServiceClass addService(Service service) {
        this.services.add(service);
        service.setServiceclass(this);
        return this;
    }

    public ServiceClass removeService(Service service) {
        this.services.remove(service);
        service.setServiceclass(null);
        return this;
    }

    public void setServices(Set<Service> services) {
        this.services.clear();
        if (services != null) {        
            this.services.addAll(services);
        } 
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        ServiceClass serviceClass = (ServiceClass) o;
        if (serviceClass.getId() == null || getId() == null) {
            return false;
        }
        return Objects.equals(getId(), serviceClass.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
        return "ServiceClass{" +
            "id=" + getId() +
            ", serviceclassname='" + getServiceclassname() + "'" +
            ", editable='" + isEditable() + "'" +
            ", deleteddate='" + getDeleteddate() + "'" +
            "}";
    }
}
