/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;
import eu.decideh2020.acsmi.backend.services.server.domain.ServiceClassTypicalAttribute;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceClassTypicalAttributeRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;

import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing ServiceClassTypicalAttribute.
 */
@RestController
@RequestMapping("/api")
public class ServiceClassTypicalAttributeResource {

    private final Logger log = LoggerFactory.getLogger(ServiceClassTypicalAttributeResource.class);

    private static final String ENTITY_NAME = "serviceClassTypicalAttribute";
        
    private final ServiceClassTypicalAttributeRepository serviceClassTypicalAttributeRepository;

    public ServiceClassTypicalAttributeResource(ServiceClassTypicalAttributeRepository serviceClassTypicalAttributeRepository) {
        this.serviceClassTypicalAttributeRepository = serviceClassTypicalAttributeRepository;
    }
    

    /**
     * GET  /service-class-typical-attributes : get all the ServiceClassTypicalAttributes.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of ServiceClassTypicalAttributes in body
     */
    @GetMapping("/service-class-typical-attributes")
    @Timed
    public List<ServiceClassTypicalAttribute> getAllServiceClassTypicalAttributes() {
       log.debug("REST request to get all ServiceClassTypicalAttributes");
        try {
        	log.debug("getAllServiceClassTypicalAttributes  ::  getting all ServiceClassTypicalAttributes...");
        	return serviceClassTypicalAttributeRepository.findAll();
        } catch (Exception e) {
        	log.error("getAllServiceClassTypicalAttributes  ::  Exception");
        	return null;
        }
    }

    /**
     * GET  /service-class-typical-attributes/:id : get the "id" ServiceTypeTypicalAttributes.
     *
     * @param id the id of the ServiceClassTypicalAttributes to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the ServiceClassTypicalAttributes, or with status 404 (Not Found)
     */
    @GetMapping("/service-class-typical-attributes/{id}")
    @Timed
    public ResponseEntity<ServiceClassTypicalAttribute> getServiceClassTypicalAttribute(@PathVariable Long id) {
        log.debug("REST request to get ServiceClassTypicalAttribute : {}", id);
        ServiceClassTypicalAttribute serviceClassTypicalAttribute = null;
        try {
        	log.debug("getServiceClassTypicalAttribute  ::  getting ServiceClassTypicalAttribute...");
        	serviceClassTypicalAttribute = serviceClassTypicalAttributeRepository.findOne(id);
        	
        	log.debug("getServiceClassTypicalAttribute  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceClassTypicalAttribute));
        } catch (Exception e) {
        	log.error("getServiceClassTypicalAttribute  ::  Exception");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassTypicalAttributeGetError", "No puede obtenerse el atributo tipico de la clase de servicio"))
                    .body(null);
        }
    }
}
