/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';

    angular
        .module('acsmiApp')
        .config(stateConfig);

    stateConfig.$inject = ['$stateProvider'];

    function stateConfig($stateProvider) {
        $stateProvider
        .state('service-class', {
            parent: 'entity',
            url: '/service-class',
            data: {
            	authorities: ['ROLE_ADMIN'],
                pageTitle: 'acsmiApp.serviceClesses.home.title'
            },
            views: {
                'content@': {
                    templateUrl: 'app/entities/service-class/service-classes.html',
                    controller: 'ServiceClassController',
                    controllerAs: 'vm'
                }
            },
            resolve: {
                translatePartialLoader: ['$translate', '$translatePartialLoader', function ($translate, $translatePartialLoader) {
                    $translatePartialLoader.addPart('serviceClasses');
                    $translatePartialLoader.addPart('global');
                    return $translate.refresh();
                }]
            }
        })
        .state('service-class-detail', {
            parent: 'service-class',
            url: '/service-class/{id}',
            data: {
            	authorities: ['ROLE_ADMIN'],
                pageTitle: 'acsmiApp.serviceClasses.detail.title'
            },
            views: {
                'content@': {
                    templateUrl: 'app/entities/service-class/service-class-detail.html',
                    controller: 'ServiceClassDetailController',
                    controllerAs: 'vm'
                }
            },
            resolve: {
                translatePartialLoader: ['$translate', '$translatePartialLoader', function ($translate, $translatePartialLoader) {
                    $translatePartialLoader.addPart('serviceClasses');
                    return $translate.refresh();
                }],
                entity: ['$stateParams', 'ServiceClass', function($stateParams, ServiceClass) {
                    return ServiceClass.get({id : $stateParams.id}).$promise;
                }],
                previousState: ["$state", function ($state) {
                    var currentStateData = {
                        name: $state.current.name || 'service-class',
                        params: $state.params,
                        url: $state.href($state.current.name, $state.params)
                    };
                    return currentStateData;
                }]
            }
        })
        .state('service-class-detail.edit', {
            parent: 'service-class-detail',
            url: '/detail/edit',
            data: {
            	authorities: ['ROLE_ADMIN']
            },
            onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                $uibModal.open({
                    templateUrl: 'app/entities/service-class/service-class-dialog.html',
                    controller: 'ServiceClassDialogController',
                    controllerAs: 'vm',
                    backdrop: 'static',
                    size: 'lg',
                    resolve: {
                        entity: ['ServiceClass', function(ServiceClass) {
                            return ServiceClass.get({id : $stateParams.id}).$promise;
                        }]
                    }
                }).result.then(function() {
                    $state.go('^', {}, { reload: false });
                }, function() {
                    $state.go('^');
                });
            }]
        })
        .state('service-class.new', {
            parent: 'service-class',
            url: '/new',
            data: {
            	authorities: ['ROLE_ADMIN']
            },
            onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                $uibModal.open({
                    templateUrl: 'app/entities/service-class/service-class-dialog.html',
                    controller: 'ServiceClassDialogController',
                    controllerAs: 'vm',
                    backdrop: 'static',
                    size: 'lg',
                    resolve: {
                        entity: function () {
                            return {
                            	serviceclassname: null,
                            	editable: false,
                                deleteddate: null,
                                id: null
                            };
                        }
                    }
                }).result.then(function() {
                    $state.go('service-class', null, { reload: 'service-class' });
                }, function() {
                    $state.go('service-class');
                });
            }]
        })
        .state('service-class.edit', {
            parent: 'service-class',
            url: '/{id}/edit',
            data: {
            	authorities: ['ROLE_ADMIN']
            },
            onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                $uibModal.open({
                    templateUrl: 'app/entities/service-class/service-class-dialog.html',
                    controller: 'ServiceClassDialogController',
                    controllerAs: 'vm',
                    backdrop: 'static',
                    size: 'lg',
                    resolve: {
                        entity: ['ServiceClass', function(ServiceClass) {
                            return ServiceClass.get({id : $stateParams.id}).$promise;
                        }]
                    }
                }).result.then(function() {
                    $state.go('service-class', null, { reload: 'service-class' });
                }, function() {
                    $state.go('^');
                });
            }]
        })
        .state('service-class.reactivate', {
            parent: 'service-class',
            url: '/{id}/reactivate',
            data: {
            	authorities: ['ROLE_ADMIN']
            },
            onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                $uibModal.open({
                    templateUrl: 'app/entities/service-class/service-class-reactivate-dialog.html',
                    controller: 'ServiceClassReactivateDialogController',
                    controllerAs: 'vm',
                    backdrop: 'static',
                    size: 'lg',
                    resolve: {
                        entity: ['ServiceClass', function(ServiceClass) {
                            return ServiceClass.get({id : $stateParams.id}).$promise;
                        }]
                    }
                }).result.then(function() {
                    $state.go('service-class', null, { reload: 'service-class' });
                }, function() {
                    $state.go('^');
                });
            }]
        })
        .state('service-class.softdelete', {
            parent: 'service-class',
            url: '/{id}/softdelete',
            data: {
            	authorities: ['ROLE_ADMIN']
            },
            onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                $uibModal.open({
                    templateUrl: 'app/entities/service-class/service-class-softdelete-dialog.html',
                    controller: 'ServiceClassSoftDeleteController',
                    controllerAs: 'vm',
                    size: 'md',
                    resolve: {
                        entity: ['ServiceClass', function(ServiceClass) {
                            return ServiceClass.get({id : $stateParams.id}).$promise;
                        }]
                    }
                }).result.then(function() {
                    $state.go('service-class', null, { reload: 'service-class' });
                }, function() {
                    $state.go('^');
                });
            }]
        })
        .state('service-class.delete', {
            parent: 'service-class',
            url: '/{id}/delete',
            data: {
            	authorities: ['ROLE_ADMIN']
            },
            onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                $uibModal.open({
                    templateUrl: 'app/entities/service-class/service-class-delete-dialog.html',
                    controller: 'ServiceClassDeleteController',
                    controllerAs: 'vm',
                    size: 'md',
                    resolve: {
                        entity: ['ServiceClass', function(ServiceClass) {
                            return ServiceClass.get({id : $stateParams.id}).$promise;
                        }]
                    }
                }).result.then(function() {
                    $state.go('service-class', null, { reload: 'service-class' });
                }, function() {
                    $state.go('^');
                });
            }]
        });
    }

})();
