/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';

    angular
        .module('acsmiApp')
        .config(stateConfig);

    stateConfig.$inject = ['$stateProvider'];

    function stateConfig($stateProvider) {
        $stateProvider
            .state('provider-members', {
                parent: 'app',
                url: '/provider-members',
                data: {
                    authorities: [
                      'ROLE_ADMIN',
                      'ROLE_CSP_USER',
                      'ROLE_PA_USER',
                      'ROLE_CSP_OWNER',
                      'ROLE_PA_OWNER',
                      'ROLE_CSP_EXTERNAL'
                    ]
                },
                views: {
                    'content@': {
                        templateUrl: 'app/provider/provider-members.html',
                        controller: 'ProviderMembersController',
                        controllerAs: 'vm'
                    }
                },
                params: {
                    page: {
                        value: '1',
                        squash: true
                    },
                    sort: {
                        value: 'id,asc',
                        squash: true
                    }
                },
                resolve: {
                    pagingParams: ['$stateParams', 'PaginationUtil', function ($stateParams, PaginationUtil) {
                        return {
                            page: PaginationUtil.parsePage($stateParams.page),
                            sort: $stateParams.sort,
                            predicate: PaginationUtil.parsePredicate($stateParams.sort),
                            ascending: PaginationUtil.parseAscending($stateParams.sort)
                        };
                    }],
                    translatePartialLoader: ['$translate', '$translatePartialLoader', function ($translate,$translatePartialLoader) {
                        $translatePartialLoader.addPart('provider');
                        return $translate.refresh();
                    }]
                }
            })
            .state('provider-members.new', {
                url: '/new',
                data: {
                    authorities: [
                      'ROLE_ADMIN',
                      'ROLE_CSP_OWNER',
                      'ROLE_PA_OWNER'
                    ]
                },
                onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                    $uibModal.open({
                        templateUrl: 'app/provider/provider-members-dialog.html',
                        controller: 'ProviderMembersDialogController',
                        controllerAs: 'vm',
                        backdrop: 'static',
                        size: 'lg',
                        resolve: {
                            entity: function () {
                                return {
                                  id: null, login: null, firstName: null, lastName: null, email: null,
                                  company: null, usersGroup:null, userType: null, creditCardType: null,
                                  creditCard: null, cifOrVat: null, activated: true, langKey: null,
                                  createdBy: null, createdDate: null, lastModifiedBy: null,
                                  lastModifiedDate: null, resetDate: null, resetKey: null, authorities: null
                                };
                            }
                        }
                    }).result.then(function() {
                        $state.go('provider-members', null, { reload: true });
                    }, function() {
                        $state.go('provider-members');
                    });
                }]
            })
            .state('provider-members.edit', {
                url: '/{login}/edit',
                data: {
                    authorities: [
                      'ROLE_ADMIN',
                      'ROLE_CSP_OWNER',
                      'ROLE_PA_OWNER'
                    ]
                },
                onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                    $uibModal.open({
                        templateUrl: 'app/provider/provider-members-dialog.html',
                        controller: 'ProviderMembersDialogController',
                        controllerAs: 'vm',
                        backdrop: 'static',
                        size: 'lg',
                        resolve: {
                            entity: ['User', function(User) {
                                return User.get({login : $stateParams.login});
                            }]
                        }
                    }).result.then(function() {
                        $state.go('provider-members', null, { reload: true });
                    }, function() {
                        $state.go('^');
                    });
                }]
            })
            .state('provider-members.delete', {
                url: '/{login}/delete',
                data: {
                    authorities: [
                      'ROLE_ADMIN',
                      'ROLE_CSP_OWNER',
                      'ROLE_PA_OWNER'
                    ]
                },
                onEnter: ['$stateParams', '$state', '$uibModal', function($stateParams, $state, $uibModal) {
                    $uibModal.open({
                        templateUrl: 'app/provider/provider-members-delete-dialog.html',
                        controller: 'ProviderMembersDeleteController',
                        controllerAs: 'vm',
                        size: 'md',
                        resolve: {
                            entity: ['User', function(User) {
                                return User.get({login : $stateParams.login});
                            }]
                        }
                    }).result.then(function() {
                        $state.go('provider-members', null, { reload: true });
                    }, function() {
                        $state.go('^');
                    });
                }]
            });
    }
})();
