/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';

    angular
        .module('acsmiApp')
        .controller('SearchController', SearchController);

    SearchController.$inject = ['$scope', 'Principal', '$translate', '$state', '$stateParams', 'ServiceClass', 'Service', 'ServiceAttributeType', 'ServiceClassTypicalAttribute'];

    function SearchController ($scope, Principal, $translate, $state, $stateParams, ServiceClass, Service, ServiceAttributeType, ServiceClassTypicalAttribute) {
        var vm = this;
        vm.service = {};
        vm.selectedAttributeType = {
        	key: null,
        	text: null,
        	value: null
        };

        vm.filter = new Array();
        vm.results = new Array();

        // Obtenemos la información del usuario logueado (en caso de estarlo)
        $scope.$on('authenticationSuccess', function() {
            getAccount();
        });

        getAccount();

        function getAccount() {
            Principal.identity().then(function(account) {
                vm.account = account;
            });
        }

        loadServiceClasses();

        // Cargar lista de clases de servicio
        function loadServiceClasses() {
        	ServiceClass.query(function(result) {
        		if (result != null && result.length > 0) {
        			// Cargar lista de tipos de atributo
        			ServiceAttributeType.query(function(resultAttr) {
        				if (resultAttr != null && resultAttr.length > 0) {
        					vm.attributeTypes = resultAttr;
        				}
        			});
        		}
        		vm.serviceClasses = result;
        	});
        }
        
        

        // Carga la lista de atributos
        $scope.loadAttributes = function() {
        	vm.aditionalAttribute = {
        		key: 0,
                name: "",
                value: "",
                mandatoryenumeration: false,
                enumerationvalues: null,
                text: ""
            };
        	vm.arraySelectedAttributes = new Array();
        	vm.arrayTypicalAttributeValues = null;

        	// Cargar atributos
        	ServiceClassTypicalAttribute.query(function(resultTypical) {
        		vm.arrayTypicalAttributes = new Array();
        		if (resultTypical != null && resultTypical.length > 0) {
        			// Cargar lista de tipos de atributos
        			ServiceAttributeType.query(function(resultAttr) {
        				vm.serviceAttributeTypes = resultAttr;
        				if (vm.serviceAttributeTypes != null && vm.serviceAttributeTypes.length > 0) {
        					// Cargar lista de atributos típicos de la clase de servicio seleccionada
        					for (var i = 0;i < resultTypical.length;i++) {
        	        			if (vm.service.serviceclass.id == resultTypical[i].serviceclassid) {
        	        				for (var j = 0;j < vm.serviceAttributeTypes.length;j++) {
        	        					if (resultTypical[i].serviceattributetypeid == vm.serviceAttributeTypes[j].id) {
        	        						resultTypical[i].attributename = vm.serviceAttributeTypes[j].serviceattributetypename;
        	        						resultTypical[i].mandatoryenumeration = vm.serviceAttributeTypes[j].mandatoryenumeration;
        	        						var obj = JSON.parse(vm.serviceAttributeTypes[j].enumerationvalues);
        	        						var arrayEnumerationValues = new Array();
        	        						for (var clave in obj) {
        	        							if (obj.hasOwnProperty(clave)) {
	        	        							var element = {
	        	        								key: clave,
	        	        								value: obj[clave]
	        	        							};
	        	        							arrayEnumerationValues.push(element);
        	        							}
        	        						}
        	        						resultTypical[i].enumerationvalues = arrayEnumerationValues;
        	        						break;
        	        					}
        	        				}
        	        				vm.arrayTypicalAttributes.push(resultTypical[i]);
        	        			}
        	        		}

        					// Cargar lista de atributos adicionales
        					vm.arrayAditionalAttributes = new Array();
		        			for (var i = 0;i < vm.serviceAttributeTypes.length;i++) {
		        				// Comprobar si el atributo figura como atributo típico de la clase del servicio
	        					for (var j = 0;j < vm.arrayTypicalAttributes.length;j++) {
	        						var found = false;
	        						if (vm.arrayTypicalAttributes[j].serviceattributetypeid == vm.serviceAttributeTypes[i].id) {
	        							found = true;
	        							break;
	        						}
	        					}
	        					// Si el atributo no figura como típico, añadirlo como adicional
	        					if (!found) {
	        						vm.arrayAditionalAttributes.push(vm.serviceAttributeTypes[i]);
		        				}
		        			}
        				}

        				// Modo edición: cargar valores de atributos
        				if (vm.service.id != null && vm.service.id != 0) {
	        				ServiceAttributeValue.query(function(resultValues) {
	        	   				if (resultValues != null && resultValues.length > 0) {
	        	   					vm.arrayTypicalAttributeValues = new Array();
	        	   					for (var i = 0;i < resultValues.length;i++) {
	        	   						if (resultValues[i].serviceid == vm.service.id) {
	        	   							// Cargar atributos típicos
	        	   							for (var j = 0;j < vm.arrayTypicalAttributes.length;j++) {
	        	   								if (vm.arrayTypicalAttributes[j].serviceattributetypeid == resultValues[i].serviceattributetypeid) {
	        	   									vm.arrayTypicalAttributeValues[resultValues[i].serviceattributetypeid] = resultValues[i].serviceattributevalue;
	        	   								}
	        	   							}

	        	   							// Cargar atributos adicionales
	        	   							var found = false;
	        	   							for (var j = 0;j < vm.arrayTypicalAttributes.length;j++) {
	        	   								if (vm.arrayTypicalAttributes[j].serviceattributetypeid == resultValues[i].serviceattributetypeid) {
	        	   									found = true;
	        	   									break;
	        	   								}
	        	   							}
	        	   							if (!found && vm.arrayAditionalAttributes != null && vm.arrayAditionalAttributes.length > 0) {
	        	   								for (var j = 0;j < vm.arrayAditionalAttributes.length;j++) {
	        	   									if (resultValues[i].serviceattributetypeid == vm.arrayAditionalAttributes[j].id) {
	        	   										var textValue = resultValues[i].serviceattributevalue;
	        	   										if (vm.arrayAditionalAttributes[j].mandatoryenumeration) {
	        	   							            	var obj = JSON.parse(vm.arrayAditionalAttributes[j].enumerationvalues);
	        	   							            	for (var clave in obj) {
	        	   												if (obj.hasOwnProperty(clave) && clave == resultValues[i].serviceattributevalue) {
	        	   													textValue = obj[clave];
	        	   													break;
	        	   												}
	        	   							            	}
	        	   						            	}
	        	   										var attr = {
	        	   											key: vm.arrayAditionalAttributes[j].id,
	                	   					                name: vm.arrayAditionalAttributes[j].serviceattributetypename,
	                	   					                value: resultValues[i].serviceattributevalue,
	                	   					                mandatoryenumeration: vm.arrayAditionalAttributes[j].mandatoryenumeration,
	                	   					                enumerationvalues: vm.arrayAditionalAttributes[j].enumerationvalues,
	                	   					                text: textValue
	                	   					            };
	        	   										vm.arraySelectedAttributes.push(attr);
	        	   										break;
	        	   									}
	        	   								}
	        	   							}
	        	   						}
	        	   					}
	        	   				}
	        	        	});
        				}
        			});
	        	}
        	});
        }


        // Función que añade un atributo adicional
        $scope.addAttribute = function() {
        	if (vm.aditionalAttribute.key != '0' && vm.aditionalAttribute.value != '') {
        		var values = vm.aditionalAttribute.key.split('#');
            	vm.aditionalAttribute.key = values[0];
            	vm.aditionalAttribute.name = values[1];
            	vm.aditionalAttribute.mandatoryenumeration = values[2] == 'true' ? true : false;
            	vm.aditionalAttribute.enumerationvalues = values[3];
            	if (!vm.aditionalAttribute.mandatoryenumeration) {
            		vm.aditionalAttribute.text = vm.aditionalAttribute.value;
            	} else {
	            	var obj = JSON.parse(vm.aditionalAttribute.enumerationvalues);
	            	for (var clave in obj) {
						if (obj.hasOwnProperty(clave) && clave == vm.aditionalAttribute.value) {
							vm.aditionalAttribute.text = obj[clave];
							break;
						}
	            	}
            	}
            	vm.arraySelectedAttributes.push(vm.aditionalAttribute);
            	vm.aditionalAttribute = {
                   	key: 0,
                   	name: "",
                   	value: "",
                   	mandatoryenumeration: false,
                    enumerationvalues: null,
                    text: ""
            	};
        	}
        }

        // Función que elimina un atributo adicional
        $scope.removeAttribute = function(index) {
        	vm.arraySelectedAttributes.splice(index, 1);
        }

        // Función de selección de un atributo adicional
        $scope.selectAditionalAttribute = function() {
        	var values = vm.aditionalAttribute.key.split('#');
        	vm.aditionalAttribute.name = values[1];
        	vm.aditionalAttribute.mandatoryenumeration = values[2] == 'true' ? true : false;
        	if (vm.aditionalAttribute.mandatoryenumeration) {
        		vm.aditionalAttribute.enumerationvalues = JSON.parse(values[3]);
        	}
        }

        // Envío de formulario
        $scope.submitSearch = function() {
        	// Construir filtro
        	var strFilter = vm.service.serviceclassid;
        	if (vm.filter.length > 0) {
        		for (var i = 0;i < vm.filter.length;i++) {
        			strFilter += '%#%<' + vm.filter[i].key + '>:<' + vm.filter[i].value + '>';
        		}
        	}

    		// Ejecutar consulta
        	Service.query({filter: strFilter}, function(result) {
        		vm.results = result;
        	});
        }


        // Cambio de clase seleccionada
        $scope.changeClass = function() {
        	vm.filter = new Array();
        	$scope.submitSearch();
        }

        // Función que añade un tipo de atributo
        $scope.addAttributeType = function() {
        	if (vm.selectedAttributeType.key != null && vm.selectedAttributeType.value != null && vm.selectedAttributeType.value != "") {
        		var values = vm.selectedAttributeType.key.split('#');
        		vm.selectedAttributeType.key = values[0];
        		vm.selectedAttributeType.text = values[1];

        		// Comprobar si existe la entrada del filtro
        		var found = false;
       			for (var i = 0;i < vm.filter.length;i++) {
       				if (vm.filter[i].key == vm.selectedAttributeType.key) {
       					found = true;
       					vm.filter[i].value = vm.selectedAttributeType.value;
       				}
       			}
       			if (!found) {
       				vm.filter.push(vm.selectedAttributeType);
       			}

       			// Reinicar objeto
       			vm.selectedAttributeType = {
       		        key: null,
       		        text: null,
       		        value: null
       			};

       			// Ejecutar consulta
       			$scope.submitSearch();
        	}
        }


        // Función que elimina un tipo de atributo
        $scope.removeAttributeType = function(index) {
        	vm.filter.splice(index, 1);

        	// Ejecutar consulta
   			$scope.submitSearch();
        }

        // Función de creación de un servicio
        $scope.submitSaveService = function() {
        	// Construir attributos a insertar/modificar
        	var array = new Array();

        	// Añadir atributos típicos
        	if (vm.arrayTypicalAttributes != null && vm.arrayTypicalAttributes.length > 0 && vm.arrayTypicalAttributeValues != null) {
        		for (var key in vm.arrayTypicalAttributeValues) {
        			if (vm.arrayTypicalAttributeValues.hasOwnProperty(key)) {
        				array.push(key);
        				array.push(vm.arrayTypicalAttributeValues[key]);
        			}
        		}
        	}

        	// Añadir atributos adicionales
        	if (vm.arraySelectedAttributes != null && vm.arraySelectedAttributes.length > 0) {
        		for (var i = 0;i < vm.arraySelectedAttributes.length;i++) {
        			array.push(vm.arraySelectedAttributes[i].key);
        			array.push(vm.arraySelectedAttributes[i].value);
        		}
        	}

        	// Si el servicio no tiene atributos, enviar un atributo ficticio para que funcione el guardado
        	if (array.length == 0) {
            	array.push("0");
        	}

        	// Guardar
            Service.save({attributes: array}, vm.service, onSaveServiceSuccess, onSaveServiceError);
        }

        // Función de éxito en alta/modificación de servicio
        function onSaveServiceSuccess(result) {
        	vm.service = null;
        	vm.arrayTypicalAttributes = null;
        	vm.arrayAditionalAttributes = null;
        }


        // Función de error en alta/modificación de servicio
        function onSaveServiceError() {}
    }
})();
